@@ -12,12 +12,11 @@ from datetime import datetime, timedelta
 import warnings
 from dateutil.parser import parse
 from dateutil.tz import tzutc
-from inspect import getargspec
+from inspect import getargspec, getmembers
 from pynamodb.constants import (
     STRING, STRING_SHORT, NUMBER, BINARY, UTC, DATETIME_FORMAT, BINARY_SET, STRING_SET, NUMBER_SET,
     MAP, MAP_SHORT, LIST, LIST_SHORT, DEFAULT_ENCODING, BOOLEAN, ATTR_TYPE_MAP, NUMBER_SHORT, NULL, SHORT_ATTR_TYPES
 )
-from pynamodb.compat import getmembers_issubclass, timedelta_total_seconds
 from pynamodb.expressions.operand import Path
 import collections
 
@@ -94,7 +93,7 @@ class Attribute(object):
 
     def __iter__(self):
         # Because we define __getitem__ below for condition expression support
-        raise TypeError("'{0}' object is not iterable".format(self.__class__.__name__))
+        raise TypeError("'{}' object is not iterable".format(self.__class__.__name__))
 
     # Condition Expression Support
     def __eq__(self, other):
@@ -197,7 +196,7 @@ class AttributeContainerMeta(type):
         cls._attributes = {}
         cls._dynamo_to_python_attrs = {}
 
-        for name, attribute in getmembers_issubclass(cls, Attribute):
+        for name, attribute in getmembers(cls, lambda o: isinstance(o, Attribute)):
             initialized = False
             if isinstance(attribute, MapAttribute):
                 # MapAttribute instances that are class attributes of an AttributeContainer class
@@ -280,7 +279,7 @@ class AttributeContainer(object):
         """
         for attr_name, attr_value in six.iteritems(attributes):
             if attr_name not in self.get_attributes():
-                raise ValueError("Attribute {0} specified does not exist".format(attr_name))
+                raise ValueError("Attribute {} specified does not exist".format(attr_name))
             setattr(self, attr_name, attr_value)
 
     def __eq__(self, other):
@@ -317,7 +316,7 @@ class SetMixin(object):
         Deserializes a set
         """
         if value and len(value):
-            return set([json.loads(val) for val in value])
+            return {json.loads(val) for val in value}
 
 
 class BinaryAttribute(Attribute):
@@ -364,9 +363,9 @@ class BinarySetAttribute(SetMixin, Attribute):
         """
         try:
             if value and len(value):
-                return set([b64decode(val.decode(DEFAULT_ENCODING)) for val in value])
+                return {b64decode(val.decode(DEFAULT_ENCODING)) for val in value}
         except AttributeError:
-            return set([b64decode(val) for val in value])
+            return {b64decode(val) for val in value}
 
 
 class UnicodeSetAttribute(SetMixin, Attribute):
@@ -402,7 +401,7 @@ class UnicodeSetAttribute(SetMixin, Attribute):
 
     def deserialize(self, value):
         if value and len(value):
-            return set([self.element_deserialize(val) for val in value])
+            return {self.element_deserialize(val) for val in value}
 
 
 class UnicodeAttribute(Attribute):
@@ -450,39 +449,6 @@ class JSONAttribute(Attribute):
         return json.loads(value, strict=False)
 
 
-class LegacyBooleanAttribute(Attribute):
-    """
-    A class for legacy boolean attributes
-
-    Previous versions of this library serialized bools as numbers.
-    This class allows you to continue to use that functionality.
-    """
-
-    attr_type = NUMBER
-
-    def serialize(self, value):
-        if value is None:
-            return None
-        elif value:
-            return json.dumps(1)
-        else:
-            return json.dumps(0)
-
-    def deserialize(self, value):
-        return bool(json.loads(value))
-
-    def get_value(self, value):
-        # we need this for the period in which you are upgrading
-        # you can switch all BooleanAttributes to LegacyBooleanAttributes
-        # this can read both but serializes as Numbers
-        # once you've transitioned, you can then switch back to
-        # BooleanAttribute and it will serialize the new fancy way
-        value_to_deserialize = super(LegacyBooleanAttribute, self).get_value(value)
-        if value_to_deserialize is None:
-            value_to_deserialize = json.dumps(value.get(BOOLEAN, 0))
-        return value_to_deserialize
-
-
 class BooleanAttribute(Attribute):
     """
     A class for boolean attributes
@@ -500,14 +466,6 @@ class BooleanAttribute(Attribute):
     def deserialize(self, value):
         return bool(value)
 
-    def get_value(self, value):
-        # we need this for legacy compatibility.
-        # previously, BOOL was serialized as N
-        value_to_deserialize = super(BooleanAttribute, self).get_value(value)
-        if value_to_deserialize is None:
-            value_to_deserialize = json.loads(value.get(NUMBER_SHORT, '0'))
-        return value_to_deserialize
-
 
 class NumberSetAttribute(SetMixin, Attribute):
     """
@@ -536,6 +494,38 @@ class NumberAttribute(Attribute):
         return json.loads(value)
 
 
+class VersionAttribute(NumberAttribute):
+    """
+    A version attribute
+    """
+    null = True
+
+    def __set__(self, instance, value):
+        """
+        Cast assigned value to int.
+        """
+        super(VersionAttribute, self).__set__(instance, int(value))
+
+    def __get__(self, instance, owner):
+        """
+        Cast retrieved value to int.
+        """
+        val = super(VersionAttribute, self).__get__(instance, owner)
+        return int(val) if isinstance(val, float) else val
+
+    def serialize(self, value):
+        """
+        Cast value to int then encode as JSON
+        """
+        return super(VersionAttribute, self).serialize(int(value))
+
+    def deserialize(self, value):
+        """
+        Decode numbers from JSON and cast to int.
+        """
+        return int(super(VersionAttribute, self).deserialize(value))
+
+
 class TTLAttribute(Attribute):
     """
     A time-to-live attribute that signifies when the item expires and can be automatically deleted.
@@ -545,22 +535,27 @@ class TTLAttribute(Attribute):
     """
     attr_type = NUMBER
 
-    def __set__(self, instance, value):
+    def _normalize(self, value):
         """
-        Converts assigned values to a UTC datetime
+        Converts value to a UTC datetime
         """
+        if value is None:
+            return
         if isinstance(value, timedelta):
-            value = int(time.time() + timedelta_total_seconds(value))
+            value = int(time.time() + value.total_seconds())
         elif isinstance(value, datetime):
             if value.tzinfo is None:
                 raise ValueError("datetime must be timezone-aware")
             value = calendar.timegm(value.utctimetuple())
-        elif value is not None:
+        else:
             raise ValueError("TTLAttribute value must be a timedelta or datetime")
-        attr_name = instance._dynamo_to_python_attrs.get(self.attr_name, self.attr_name)
-        if value is not None:
-            value = datetime.utcfromtimestamp(value).replace(tzinfo=tzutc())
-        instance.attribute_values[attr_name] = value
+        return datetime.utcfromtimestamp(value).replace(tzinfo=tzutc())
+
+    def __set__(self, instance, value):
+        """
+        Converts assigned values to a UTC datetime
+        """
+        super(TTLAttribute, self).__set__(instance, self._normalize(value))
 
     def serialize(self, value):
         """
@@ -568,7 +563,7 @@ class TTLAttribute(Attribute):
         """
         if value is None:
             return None
-        return json.dumps(calendar.timegm(value.utctimetuple()))
+        return json.dumps(calendar.timegm(self._normalize(value).utctimetuple()))
 
     def deserialize(self, value):
         """
@@ -597,13 +592,16 @@ class UTCDateTimeAttribute(Attribute):
         """
         Takes a UTC datetime string and returns a datetime object
         """
-        # First attempt to parse the datetime with the datetime format used
-        # by default when storing UTCDateTimeAttributes.  This is signifantly
-        # faster than always going through dateutil.
         try:
-            return datetime.strptime(value, DATETIME_FORMAT)
-        except ValueError:
-            return parse(value)
+            return _fast_parse_utc_datestring(value)
+        except (ValueError, IndexError):
+            try:
+                # Attempt to parse the datetime with the datetime format used
+                # by default when storing UTCDateTimeAttributes.  This is signifantly
+                # faster than always going through dateutil.
+                return datetime.strptime(value, DATETIME_FORMAT)
+            except ValueError:
+                return parse(value)
 
 
 class NullAttribute(Attribute):
@@ -616,13 +614,6 @@ class NullAttribute(Attribute):
         return None
 
 
-class MapAttributeMeta(AttributeContainerMeta):
-    """
-    This is only here for backwards compatibility: i.e. so type(MapAttribute) == MapAttributeMeta
-    """
-
-
-@add_metaclass(MapAttributeMeta)
 class MapAttribute(Attribute, AttributeContainer):
     """
     A Map Attribute
@@ -686,7 +677,7 @@ class MapAttribute(Attribute, AttributeContainer):
 
     def __init__(self, **attributes):
         # Store the kwargs used by Attribute.__init__ in case `_make_attribute` is called.
-        self.attribute_kwargs = dict((arg, attributes.pop(arg)) for arg in self.attribute_args if arg in attributes)
+        self.attribute_kwargs = {arg: attributes.pop(arg) for arg in self.attribute_args if arg in attributes}
 
         # Assume all instances should behave like an AttributeContainer. Instances that are intended to be
         # used as Attributes will be transformed by AttributeContainerMeta during creation of the containing class.
@@ -763,17 +754,17 @@ class MapAttribute(Attribute, AttributeContainer):
         elif item in self._attributes:
             return getattr(self, item)
         else:
-            raise AttributeError("'{0}' has no attribute '{1}'".format(self.__class__.__name__, item))
+            raise AttributeError("'{}' has no attribute '{}'".format(self.__class__.__name__, item))
 
     def __setitem__(self, item, value):
         if not self._is_attribute_container():
-            raise TypeError("'{0}' object does not support item assignment".format(self.__class__.__name__))
+            raise TypeError("'{}' object does not support item assignment".format(self.__class__.__name__))
         if self.is_raw():
             self.attribute_values[item] = value
         elif item in self._attributes:
             setattr(self, item, value)
         else:
-            raise AttributeError("'{0}' has no attribute '{1}'".format(self.__class__.__name__, item))
+            raise AttributeError("'{}' has no attribute '{}'".format(self.__class__.__name__, item))
 
     def __getattr__(self, attr):
         # This should only be called for "raw" (i.e. non-subclassed) MapAttribute instances.
@@ -783,7 +774,7 @@ class MapAttribute(Attribute, AttributeContainer):
                 return self.attribute_values[attr]
             except KeyError:
                 pass
-        raise AttributeError("'{0}' has no attribute '{1}'".format(self.__class__.__name__, attr))
+        raise AttributeError("'{}' has no attribute '{}'".format(self.__class__.__name__, attr))
 
     def __setattr__(self, name, value):
         # "Raw" (i.e. non-subclassed) instances set their name-value pairs in the `attribute_values` dictionary.
@@ -814,7 +805,7 @@ class MapAttribute(Attribute, AttributeContainer):
         if can_be_null and value is None:
             return True
         if getattr(self, key) is None:
-            raise ValueError("Attribute '{0}' cannot be None".format(key))
+            raise ValueError("Attribute '{}' cannot be None".format(key))
         return True  # TODO: check that the actual type of `value` meets requirements of `attr`
 
     def validate(self):
@@ -934,6 +925,26 @@ def _get_key_for_serialize(value):
     return SERIALIZE_KEY_MAP[value_type]
 
 
+def _fast_parse_utc_datestring(datestring):
+    # Method to quickly parse strings formatted with '%Y-%m-%dT%H:%M:%S.%f+0000'.
+    # This is ~5.8x faster than using strptime and 38x faster than dateutil.parser.parse.
+    _int = int  # Hack to prevent global lookups of int, speeds up the function ~10%
+    try:
+        if (datestring[4] != '-' or datestring[7] != '-' or datestring[10] != 'T' or
+                datestring[13] != ':' or datestring[16] != ':' or datestring[19] != '.' or
+                datestring[-5:] != '+0000'):
+            raise ValueError("Datetime string '{}' does not match format "
+                            "'%Y-%m-%dT%H:%M:%S.%f+0000'".format(datestring))
+        return datetime(
+            _int(datestring[0:4]), _int(datestring[5:7]), _int(datestring[8:10]),
+            _int(datestring[11:13]), _int(datestring[14:16]), _int(datestring[17:19]),
+            _int(round(float(datestring[19:-5]) * 1e6)), tzutc()
+        )
+    except (TypeError, ValueError):
+        raise ValueError("Datetime string '{}' does not match format "
+                         "'%Y-%m-%dT%H:%M:%S.%f+0000'".format(datestring))
+
+
 class ListAttribute(Attribute):
     attr_type = LIST
     element_type = None
