@@ -3,6 +3,7 @@ PynamoDB Connection classes
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~
 """
 from pynamodb.connection.base import Connection
+from pynamodb.constants import DEFAULT_BILLING_MODE, KEY
 
 
 class TableConnection(object):
@@ -14,10 +15,12 @@ class TableConnection(object):
                  table_name,
                  region=None,
                  host=None,
-                 session_cls=None,
-                 request_timeout_seconds=None,
+                 connect_timeout_seconds=None,
+                 read_timeout_seconds=None,
                  max_retry_attempts=None,
                  base_backoff_ms=None,
+                 max_pool_connections=None,
+                 extra_headers=None,
                  aws_access_key_id=None,
                  aws_secret_access_key=None):
         self._hash_keyname = None
@@ -25,10 +28,12 @@ class TableConnection(object):
         self.table_name = table_name
         self.connection = Connection(region=region,
                                      host=host,
-                                     session_cls=session_cls,
-                                     request_timeout_seconds=request_timeout_seconds,
+                                     connect_timeout_seconds=connect_timeout_seconds,
+                                     read_timeout_seconds=read_timeout_seconds,
                                      max_retry_attempts=max_retry_attempts,
-                                     base_backoff_ms=base_backoff_ms)
+                                     base_backoff_ms=base_backoff_ms,
+                                     max_pool_connections=max_pool_connections,
+                                     extra_headers=extra_headers)
 
         if aws_access_key_id and aws_secret_access_key:
             self.connection.session.set_credentials(aws_access_key_id,
@@ -40,11 +45,39 @@ class TableConnection(object):
         """
         return self.connection.get_meta_table(self.table_name, refresh=refresh)
 
-    def delete_item(self, hash_key,
+    def get_operation_kwargs(self,
+                             hash_key,
+                             range_key=None,
+                             key=KEY,
+                             attributes=None,
+                             attributes_to_get=None,
+                             actions=None,
+                             condition=None,
+                             consistent_read=None,
+                             return_values=None,
+                             return_consumed_capacity=None,
+                             return_item_collection_metrics=None,
+                             return_values_on_condition_failure=None):
+        return self.connection.get_operation_kwargs(
+            self.table_name,
+            hash_key,
+            range_key=range_key,
+            key=key,
+            attributes=attributes,
+            attributes_to_get=attributes_to_get,
+            actions=actions,
+            condition=condition,
+            consistent_read=consistent_read,
+            return_values=return_values,
+            return_consumed_capacity=return_consumed_capacity,
+            return_item_collection_metrics=return_item_collection_metrics,
+            return_values_on_condition_failure=return_values_on_condition_failure
+        )
+
+    def delete_item(self,
+                    hash_key,
                     range_key=None,
                     condition=None,
-                    expected=None,
-                    conditional_operator=None,
                     return_values=None,
                     return_consumed_capacity=None,
                     return_item_collection_metrics=None):
@@ -56,8 +89,6 @@ class TableConnection(object):
             hash_key,
             range_key=range_key,
             condition=condition,
-            expected=expected,
-            conditional_operator=conditional_operator,
             return_values=return_values,
             return_consumed_capacity=return_consumed_capacity,
             return_item_collection_metrics=return_item_collection_metrics)
@@ -66,10 +97,7 @@ class TableConnection(object):
                     hash_key,
                     range_key=None,
                     actions=None,
-                    attribute_updates=None,
                     condition=None,
-                    expected=None,
-                    conditional_operator=None,
                     return_consumed_capacity=None,
                     return_item_collection_metrics=None,
                     return_values=None
@@ -82,20 +110,16 @@ class TableConnection(object):
             hash_key,
             range_key=range_key,
             actions=actions,
-            attribute_updates=attribute_updates,
             condition=condition,
-            expected=expected,
-            conditional_operator=conditional_operator,
             return_consumed_capacity=return_consumed_capacity,
             return_item_collection_metrics=return_item_collection_metrics,
             return_values=return_values)
 
-    def put_item(self, hash_key,
+    def put_item(self,
+                 hash_key,
                  range_key=None,
                  attributes=None,
                  condition=None,
-                 expected=None,
-                 conditional_operator=None,
                  return_values=None,
                  return_consumed_capacity=None,
                  return_item_collection_metrics=None):
@@ -108,8 +132,6 @@ class TableConnection(object):
             range_key=range_key,
             attributes=attributes,
             condition=condition,
-            expected=expected,
-            conditional_operator=conditional_operator,
             return_values=return_values,
             return_consumed_capacity=return_consumed_capacity,
             return_item_collection_metrics=return_item_collection_metrics)
@@ -151,52 +173,10 @@ class TableConnection(object):
             consistent_read=consistent_read,
             attributes_to_get=attributes_to_get)
 
-    def rate_limited_scan(
-             self,
-             filter_condition=None,
-             attributes_to_get=None,
-             page_size=None,
-             limit=None,
-             conditional_operator=None,
-             scan_filter=None,
-             segment=None,
-             total_segments=None,
-             exclusive_start_key=None,
-             timeout_seconds=None,
-             read_capacity_to_consume_per_second=None,
-             allow_rate_limited_scan_without_consumed_capacity=None,
-             max_sleep_between_retry=None,
-             max_consecutive_exceptions=None,
-             consistent_read=None,
-             index_name=None):
-        """
-        Performs the scan operation with rate limited
-        """
-        return self.connection.rate_limited_scan(
-            self.table_name,
-            filter_condition=filter_condition,
-            attributes_to_get=attributes_to_get,
-            page_size=page_size,
-            limit=limit,
-            conditional_operator=conditional_operator,
-            scan_filter=scan_filter,
-            segment=segment,
-            total_segments=total_segments,
-            exclusive_start_key=exclusive_start_key,
-            timeout_seconds=timeout_seconds,
-            read_capacity_to_consume_per_second=read_capacity_to_consume_per_second,
-            allow_rate_limited_scan_without_consumed_capacity=allow_rate_limited_scan_without_consumed_capacity,
-            max_sleep_between_retry=max_sleep_between_retry,
-            max_consecutive_exceptions=max_consecutive_exceptions,
-            consistent_read=consistent_read,
-            index_name=index_name)
-
     def scan(self,
              filter_condition=None,
              attributes_to_get=None,
              limit=None,
-             conditional_operator=None,
-             scan_filter=None,
              return_consumed_capacity=None,
              segment=None,
              total_segments=None,
@@ -211,8 +191,6 @@ class TableConnection(object):
             filter_condition=filter_condition,
             attributes_to_get=attributes_to_get,
             limit=limit,
-            conditional_operator=conditional_operator,
-            scan_filter=scan_filter,
             return_consumed_capacity=return_consumed_capacity,
             segment=segment,
             total_segments=total_segments,
@@ -228,12 +206,9 @@ class TableConnection(object):
               consistent_read=False,
               exclusive_start_key=None,
               index_name=None,
-              key_conditions=None,
-              query_filters=None,
               limit=None,
               return_consumed_capacity=None,
               scan_index_forward=None,
-              conditional_operator=None,
               select=None
               ):
         """
@@ -248,12 +223,9 @@ class TableConnection(object):
             consistent_read=consistent_read,
             exclusive_start_key=exclusive_start_key,
             index_name=index_name,
-            key_conditions=key_conditions,
-            query_filters=query_filters,
             limit=limit,
             return_consumed_capacity=return_consumed_capacity,
             scan_index_forward=scan_index_forward,
-            conditional_operator=conditional_operator,
             select=select)
 
     def describe_table(self):
@@ -294,7 +266,8 @@ class TableConnection(object):
                      write_capacity_units=None,
                      global_secondary_indexes=None,
                      local_secondary_indexes=None,
-                     stream_specification=None):
+                     stream_specification=None,
+                     billing_mode=DEFAULT_BILLING_MODE):
         """
         Performs the CreateTable operation and returns the result
         """
@@ -306,5 +279,6 @@ class TableConnection(object):
             write_capacity_units=write_capacity_units,
             global_secondary_indexes=global_secondary_indexes,
             local_secondary_indexes=local_secondary_indexes,
-            stream_specification=stream_specification
+            stream_specification=stream_specification,
+            billing_mode=billing_mode
         )
